import {Context, DefaultState, DefaultContext, Middleware, Next} from 'koa';
import Router from 'koa-router';
import {createHash} from 'crypto';
import {promises} from 'fs'
/**
 * content file or directory referrer
 */
export interface ContentReferrer{
    getHash():Promise<string>;
    readonly path: string;
    readonly type: string;
    desc: object|undefined;
}
type ContentReferrerConstructor =  (new (path:string,desc?:object) => ContentReferrer)&{content_type:string};
export const createDefaultClass = (type:string):ContentReferrerConstructor=>{
    let cons = class implements ContentReferrer{
        readonly path: string;
        type = type;
        static content_type = type;
        
        desc: object|undefined;
        constructor(path:string,desc?:object){
            this.path = path;
            this.desc = desc;
        }
        async getHash():Promise<string>{
            const stat = await promises.stat(this.path);
            const hash = createHash("sha512");
            hash.update(stat.mode.toString());
            //if(this.desc !== undefined)
            //    hash.update(JSON.stringify(this.desc));
            hash.update(stat.size.toString());
            return hash.digest("base64");
        }
    };
    return cons;
}
let ContstructorTable:{[k:string]:ContentReferrerConstructor} = {};
export function registerContentReferrer(s: ContentReferrerConstructor){
    console.log(`registered content type: ${s.content_type}`)
    ContstructorTable[s.content_type] = s;
}
export function createContentReferrer(type:string,path:string,desc?:object){
    const constructorMethod = ContstructorTable[type];
    if(constructorMethod === undefined){
        console.log(type);
        throw new Error("undefined");
    }
    return new constructorMethod(path,desc);
}