import {sign, decode, verify} from 'jsonwebtoken';
import Koa from 'koa';
import Router from 'koa-router';
import { sendError } from './route/error_handler';
import Knex from 'knex'
import { createKnexUserController } from './db/mod';
import { request } from 'http';
import { get_setting } from './setting';
import { IUser } from './model/mod';

const loginTokenName = 'access_token'

export const createLoginMiddleware = (knex: Knex)=>{
    const userController = createKnexUserController(knex);
    return async (ctx: Koa.Context,next: Koa.Next)=>{
        const setting = get_setting();
        const secretKey = setting.jwt_secretkey;
        const body = ctx.request.body;
        if(!('username' in body)||!('password' in body)){
            sendError(400,"invalid form : username or password is not found in query.");
            return;
        }
        const username = body['username'];
        const password = body['password'];
        if(typeof username !== "string" || typeof password !== "string"){
            sendError(400,"invalid form : username or password is not string")
            return;
        }
        const user = await userController.findUser(username);
        if(user === undefined){
            sendError(401,"not authorized");
            return;
        }
        if(!user.password.check_password(password)){
            sendError(401,"not authorized");
            return;
        }
        const userPermission = await user.get_permissions();
        const payload = sign({
            username: user.username,
            permission: userPermission
        },secretKey,{expiresIn:'3d'});
        ctx.cookies.set(loginTokenName,payload,{httpOnly:true, secure: !setting.localmode,sameSite:'strict'});
        ctx.body = {ok:true, username: user.username, permission: userPermission}
        console.log(`${username} logined`);
        return;
    };
};
export const LogoutMiddleware = (ctx:Koa.Context,next:Koa.Next)=>{
    ctx.cookies.set(loginTokenName,undefined);
    ctx.body = {ok:true};
    return;
}
export const UserMiddleWare = async (ctx:Koa.Context,next:Koa.Next)=>{
    const secretKey = get_setting().jwt_secretkey;
    const payload = ctx.cookies.get(loginTokenName);
    if(payload == undefined){
        ctx.state['user'] = undefined;
        return await next();
    }
    ctx.state['user'] = verify(payload,secretKey);
    await next();
}

export const getAdmin = async(knex : Knex)=>{
    const cntr = createKnexUserController(knex);
    const admin = await cntr.findUser("admin");
    if(admin === undefined){
        throw new Error("initial process failed!");//???
    }
    return admin;
}

export const isAdminFirst = (admin: IUser)=>{
    return admin.password.hash === "unchecked" && admin.password.salt === "unchecked";
}