import {TagAccessor} from './tag';
import {check_type} from './../util/type_check'

type JSONPrimitive = null|boolean|number|string;
interface JSONMap extends Record<string, JSONType>{}
interface JSONArray extends Array<JSONType>{};
type JSONType = JSONMap|JSONPrimitive|JSONArray;

export interface ContentContent{
    title           : string,
    content_type    : string,
    basepath        : string,
    filename        : string,
    hash?           : string,
    additional      : JSONMap,
    tags            : string[],//eager loading 
}

export const MetaContentContent = {
    title         : "string",
    content_type  : "string",
    basepath      : "string",
    filename      : "string",
    hash          : "string",
    additional    : "object",
    tags          : "string[]",
}

export const isContentContent = (c : any):c is ContentContent =>{
    return check_type<ContentContent>(c,MetaContentContent);
}

export interface Content extends ContentContent{
    readonly id: number;
};

export const isContent = (c: any):c is Content =>{
    if('id' in c && typeof c['id'] === "number"){
        const {id, ...rest} = c;
        return isContentContent(rest);
    }
    return false;
}

export type QueryListOption = {
    /**
     * search word
     */
    word?:string,
    allow_tag?:string[],
    /**
     * limit of list
     * @default 20
     */
    limit?:number,
    /**
     * use offset if true, otherwise
     * @default false
     */
    use_offset?:boolean,
    /**
     * cursor of contents
     */
    cursor?:number,
    /**
     * offset of contents
     */
    offset?:number,
    /**
     * tag eager loading
     * @default true
     */
    eager_loading?:boolean,
    /**
     * content type
     */
    content_type?:string
}

export interface ContentAccessor{
    /**
     * find list by option
     * @returns content list
     */
    findList: (option?:QueryListOption)=>Promise<Content[]>,
    /**
     * @returns content if exist, otherwise undefined
     */
    findById: (id:number,tagload?:boolean)=> Promise<Content| undefined>,
    /**
     * 
     */
    findListByBasePath:(basepath: string)=>Promise<Content[]>;
    /**
     * update content except tag.
     */
    update:(c:Partial<Content> & { id:number })=>Promise<boolean>;
    /**
     * add content
     */
    add:(c:ContentContent)=>Promise<number>;
    /**
     * delete content
     * @returns if it exists, return true.
     */
    del:(id:number)=>Promise<boolean>;
    /**
     * @param c Valid Content
     * @param tagname tag name to add
     * @returns if success, return true
     */
    addTag:(c:Content,tag_name:string)=>Promise<boolean>;
    /**
     * @returns if success, return true
     */
    delTag:(c:Content,tag_name:string)=>Promise<boolean>;
};